/**
 * Created by jmoutard on 21/07/16.
 */

'use strict';

angular.module('adf.electre')
    .controller('navigationController', [
        "$scope", "$rootScope", "$q", "$location", "storeService", "loggedIn", "$timeout",
        function ($scope, $rootScope, $q, $location, storeService, loggedIn, $timeout) {
            var nav = this;
            nav.navCollapsed = false;
            nav.loggedIn = loggedIn;

            $scope.tree = [
                {
                    name: "Home",
                    link: "home"
                },{
                    name: "Log in",
                    link: "login"
                },{
                    name: "Log out",
                    link: "logout"
                }
            ];
            $scope.serverDashboardTree = [
                {
                    name: "title",
                    link: "board"
                }
            ];

            this.toggleNav = function(){
                nav.navCollapsed = !nav.navCollapsed;
            };

            this.navClass = function(page) {
                var currentRoute = $location.path().substring(1);
                console.log(page, currentRoute);
                console.log(page === currentRoute, new RegExp(page).test(currentRoute));
                return page === currentRoute || new RegExp(page).test(currentRoute) ? 'active' : '';
            };

            this.create = function(){
                var id = '_' + Date.now();
                var q = storeService.add(id, {
                    "title": (loggedIn ? "New User Layout" : "New Local Layout"),
                    "structure": '2 rows (12-(3-3-3-3))',
                    "rows" : [{
                        "columns" : [{
                            "styleClass" : 'col-md-12'
                        }]
                    },{
                        "columns" : [{
                            "styleClass" : 'col-md-3'
                        },{
                            "styleClass" : 'col-md-3'
                        },{
                            "styleClass" : 'col-md-3'
                        },{
                            "styleClass" : 'col-md-3'
                        }]
                    }]
                });

                $q.all([q, storeService.getAll()]).then(function(values){
                    takeDashboards(values[1]);

                    $location.path((loggedIn ? '/remote_board/' + id : "/local_board/localDashboard"));
                    $rootScope.$broadcast('navChanged');
                }, function (error) {
                    displayError(error);
                });
            };

            storeService.getAll().then(function(data){
                takeDashboards(data);
            });

            $scope.$on('navChanged', function(){
                resetDashboards();
                storeService.getAll().then(function(data){
                    takeDashboards(data);
                });
            });

            $scope.$on('removeDashboard', function (event, id) {
                storeService.remove(id)
                .then(function () {
                    $scope.$emit('navChanged');
                    $location.path('/');
                }, function (error) {
                    displayError(error);
                });
            });
            $scope.$on('updateDashboard', function (event, id, data) {
                storeService.add(id, data).then(function () {
                    $scope.$emit('navChanged');
                }, function (error) {
                    displayError(error);
                });
            });

            /* ROUTING ERROR */
            $rootScope.$on('$stateChangeError', function (event, toState, toParams, fromState, fromParams, error) {
                if (error && error.code) {
                    displayError(error);
                } else {
                    console.log(error);
                }
            });
            /**
             * @description Display errors from the storage service.
             * @param {object} error Error from the storage service.
             */
            function displayError (error) {
                $scope.errorMessage = (error.message ? error.message() : error);
                $timeout(function () {
                    $scope.errorMessage = false;
                }, 2500);
            }
            /**
             * @description Taking the dashboards from the storage. Creating objects for the navbar menu.
             * @param {object} values Data given by the storage.
             */
            function takeDashboards (values) {
                if (values && typeof values === "object") {
                    if (values.serverStorage && values.serverStorage.length > 0) {
                        nav.serverDashboard = true;
                        $scope.serverDashboardTree = [];
                        for (var si = 0, serverItem = values.serverStorage[si] ;
                             si < values.serverStorage.length;
                             si++, serverItem = values.serverStorage[si]
                        ) {
                            $scope.serverDashboardTree.push({"name" : serverItem.title, "link": "remote_board({id: \"" + serverItem.id + "\"})"});
                        }
                    }
                    if (values.localStorage && values.localStorage.length > 0) {
                        nav.localDashboard = true;
                        $scope.localItem = values.localStorage[0];
                        console.log(nav, $scope);
                    }
                }
            }
            /**
             * @description Removing all dashboard's object in order to clear the navbar menu.
             */
            function resetDashboards () {
                nav.localDashboard = false;
                nav.serverDashboard = false;
                $scope.serverDashboardTree = undefined;
                $scope.localItem = undefined;
            }
        }
    ]);